/* 
 * HotlinkingFilter.java
 * 
 * Created on 23.09.2003.
 *
 * Copyright (c) 2003, 2004, 2005 Johann Burkard (<mailto:jb@eaio.com>)
 * <http://eaio.com>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */
package com.eaio.j2ee.filter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * The HotlinkingFilter checks the referer header (if present) for matches of
 * a number of "good" referer strings. If no matches are found, the request is
 * redirected to another URL. If the referer header is not present, the
 * request is always accepted.
 * <p>
 * The init parameters are:
 * <ul>
 * <li><code>redirect</code> - (optional) an absolute URL that requests
 * with a "bad" referer should be redirected to. If not specified, the request
 * will be denied.</li>
 * <li>a number of <code>referer</code> parameters which are the "good"
 * referers. To use more than one "good" referer, add a number to the
 * <code>referer</code> parameters, like "referer_1", "referer_2" etc.</li>
 * </ul>
 * <p>
 * Don't forget to add your local IP address to the list of "good" referers
 * when testing locally.
 * 
 * @author <a href="mailto:jb@eaio.com">Johann Burkard</a>
 * @version $Id$
 */
public class HotlinkingFilter implements Filter {

    /**
     * The "referer" parameter name.
     */
    public static final String PARAMETER_REFERER = "referer";

    /**
     * The "redirect" parameter name.
     */
    public static final String PARAMETER_REDIRECT = "redirect";

    /**
     * The configuration.
     */
    private FilterConfig config;

    /**
     * The "good" referers.
     */
    private String[] goodReferers = new String[0];

    /**
     * The redirect URL.
     */
    private String redirectURL = "http://www.google.com";

    /**
     * Constructor for HotlinkingFilter.
     */
    public HotlinkingFilter() {}

    /**
     * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest,
     *  javax.servlet.ServletResponse, javax.servlet.FilterChain)
     */
    public void doFilter(
        ServletRequest request,
        ServletResponse response,
        FilterChain chain)
        throws IOException, ServletException {

        boolean allowed = false;

        if (request instanceof HttpServletRequest) {

            String referer =
                ((HttpServletRequest) request).getHeader(PARAMETER_REFERER);

            if (referer == null) {
                allowed = true;
            }
            else {
                for (int i = 0; i < goodReferers.length; ++i) {
                    if (referer.indexOf(goodReferers[i]) != -1) {
                        allowed = true;
                        break;
                    }
                }
            }

            if (allowed) {
                chain.doFilter(request, response);
                return;
            }

        }

        if (response instanceof HttpServletResponse) {
            ((HttpServletResponse) response).sendRedirect(redirectURL);
        }

    }

    /**
     * @see javax.servlet.Filter#getFilterConfig()
     */
    public FilterConfig getFilterConfig() {
        return config;
    }

    /**
     * @see javax.servlet.Filter#setFilterConfig(javax.servlet.FilterConfig)
     */
    public void setFilterConfig(FilterConfig config) {
        if (config == null) {
            return;
        }
        config.getServletContext().log(
            getClass().getName() + ": setFilterConfig()");
        this.config = config;
        List referers = new ArrayList();
        Enumeration keys = config.getInitParameterNames();
        while (keys.hasMoreElements()) {
            String key = (String) keys.nextElement();
            String value = config.getInitParameter(key);
            if (PARAMETER_REDIRECT.equalsIgnoreCase(key)) {
                redirectURL = value;
            }
            else if (key.toLowerCase().startsWith(PARAMETER_REFERER)) {
                referers.add(value);
            }
        }
        goodReferers = (String[]) referers.toArray(new String[referers.size()]);
        StringBuffer configuration = new StringBuffer(256);
        configuration.append(
            getClass().getName() + ": setFilterConfig() - redirect = <");
        configuration.append(redirectURL);
        configuration.append(">, referer = ['");
        for (int i = 0; i < goodReferers.length; ++i) {
            configuration.append(goodReferers[i]);
            if (i < goodReferers.length - 1) {
                configuration.append("', '");
            }
        }
        configuration.append("']");
        config.getServletContext().log(configuration.toString());
    }

    /**
     * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
     */
    public void init(FilterConfig config) throws ServletException {
        config.getServletContext().log(getClass().getName() + ": init()");
        setFilterConfig(config);
    }

    /**
     * @see javax.servlet.Filter#destroy()
     */
    public void destroy() {
        config.getServletContext().log(getClass().getName() + ": destroy()");
    }

}
