/* 
 * AbstractTestCase.java
 * 
 * Created on 11.10.2005.
 *
 * Copyright (c) 2003, 2004, 2005, 2006 Johann Burkard (<mailto:jb@eaio.com>)
 * <http://eaio.com>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */
package com.eaio.util.junit;

import java.io.*;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Hashtable;

import junit.framework.TestCase;

/**
 * A basic test superclass that does a number of static tests.
 * 
 * @author <a href="mailto:jb@eaio.com">Johann Burkard</a>
 * @version $Id$
 */
public abstract class AbstractTestCase extends TestCase {

    /**
     * Constructor for AbstractTestCase.
     */
    public AbstractTestCase() {
        super();
    }

    /**
     * Constructor for AbstractTestCase.
     * @param arg0
     */
    public AbstractTestCase(String arg0) {
        super(arg0);
    }

    /**
     * Returns the class to test. May be overridden if the class name is not the
     * name of this class minus "Test".
     * 
     * @return the test class
     */
    protected Class getTestClass() {
        String thisClazz = getClass().getName();
        try {
            return Class.forName(thisClazz.substring(0,
                    thisClazz.indexOf("Test")));
        }
        catch (ClassNotFoundException ex) {
            ex.printStackTrace();
            fail(ex.getLocalizedMessage());
        }
        catch (RuntimeException ex) {
            ex.printStackTrace();
            fail(ex.getLocalizedMessage());
        }
        return null;
    }

    /**
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        Class clazz = getTestClass();
        verifyClass(clazz);
        checkFields(clazz);
        Class[] subClasses = clazz.getClasses();
        for (int i = 0; i < subClasses.length; ++i) {
            verifyClass(subClasses[i]);
            checkFields(subClasses[i]);
        }
        subClasses = clazz.getDeclaredClasses();
        for (int i = 0; i < subClasses.length; ++i) {
            verifyClass(subClasses[i]);
            checkFields(subClasses[i]);
        }
        Field[] fields = clazz.getFields();
        for (int i = 0; i < fields.length; ++i) {
            if (fields[i].getType().getName().startsWith(clazz.getName())
                    && (fields[i].getModifiers() & Modifier.STATIC) == Modifier.STATIC) {
                verifyClass(fields[i].get(null).getClass());
                checkFields(fields[i].get(null).getClass());
            }
        }
        fields = clazz.getDeclaredFields();
        for (int i = 0; i < fields.length; ++i) {
            if (fields[i].getType().getName().startsWith(clazz.getName())
                    && (fields[i].getModifiers() & Modifier.STATIC) == Modifier.STATIC) {
                fields[i].setAccessible(true);
                verifyClass(fields[i].get(null).getClass());
                checkFields(fields[i].get(null).getClass());
            }
        }

        String packageName = clazz.getName().substring(0,
                clazz.getName().lastIndexOf('.'));
        String packageHTML = "/" + packageName.replace('.', '/')
                + "/package.html";
        assertNotNull("No 'package.html' in package '" + packageName
                + "' present - please document your packages.",
                clazz.getResource(packageHTML));
    }

    protected void checkFields(Class clazz) {
        Field[] publicFields = clazz.getFields();
        Field field;
        for (int i = 0; i < publicFields.length; ++i) {
            field = publicFields[i];
            if ((field.getModifiers() & Modifier.STATIC) == Modifier.STATIC) {
                if ((field.getModifiers() & Modifier.FINAL) != Modifier.FINAL
                        && Object.class.isAssignableFrom(field.getType())) {
                    fail("Do not use public static fields of Object types for security reasons: "
                            + field);
                }
            }
        }
    }

    protected void verifyClass(Class clazz) {
        if (Serializable.class.isAssignableFrom(clazz)) {
            try {
                Field serialVersionUID = clazz.getDeclaredField("serialVersionUID");
                assertTrue(serialVersionUID.getType() == Long.TYPE);
                assertTrue((serialVersionUID.getModifiers() & (Modifier.STATIC | Modifier.FINAL)) == (Modifier.STATIC | Modifier.FINAL));
            }
            catch (Exception ex) {
                ex.printStackTrace();
                fail("Must declare serialVersionUID (static final long) in '"
                        + clazz.getName() + "'.");
            }
        }

        if (Externalizable.class.isAssignableFrom(clazz)
                && !clazz.isInterface()) {
            assertTrue("Cannot have Externalizable inner classes.",
                    clazz.getName().indexOf('$') == -1);
            try {
                clazz.getConstructor(null);
            }
            catch (NoSuchMethodException ex) {
                fail("Must have a no-argument constructor if implementing Externalizable.");
            }
            try {
                testSerialization(clazz.newInstance());
            }
            catch (Exception ex) {
                ex.printStackTrace();
                fail(ex.toString());
            }
        }

        if (Cloneable.class.isAssignableFrom(clazz)) {
            try {
                Method clone = clazz.getMethod("clone", new Class[] {});
                assertTrue(
                        "clone method is not public in a class that implements Cloneable.",
                        (clone.getModifiers() & Modifier.PUBLIC) == Modifier.PUBLIC);
                Object instance = clazz.newInstance();
                Object clonedInstance = clone.invoke(instance, null);

                assertTrue(clonedInstance != instance);
                assertEquals(clonedInstance, instance);
                assertEquals(clonedInstance.hashCode(), instance.hashCode());
            }
            catch (Exception ex) {
                if (!(ex instanceof InstantiationException)) {
                    ex.printStackTrace();
                    fail(ex.toString());
                }
            }
        }

        try {
            Method readObject = clazz.getDeclaredMethod("readObject",
                    new Class[] { ObjectInputStream.class });
            assertTrue(readObject.getReturnType() == Void.TYPE);
            assertTrue((readObject.getModifiers() & Modifier.PRIVATE) == Modifier.PRIVATE);
        }
        catch (Exception ex) {}

        try {
            Method writeObject = clazz.getDeclaredMethod("writeObject",
                    new Class[] { ObjectOutputStream.class });
            assertTrue(writeObject.getReturnType() == Void.TYPE);
            assertTrue((writeObject.getModifiers() & Modifier.PRIVATE) == Modifier.PRIVATE);
        }
        catch (Exception ex) {}

        try {
            clazz.getDeclaredMethod("hashCode", new Class[] {});
            try {
                clazz.getDeclaredMethod("equals", new Class[] { Object.class });
            }
            catch (NoSuchMethodException ex) {
                fail("Must overwrite #equals(Object) in class '"
                        + clazz.getName() + "' when implementing #hashCode().");
            }

            Object o1 = clazz.newInstance();
            Object o2 = clazz.newInstance();

            assertTrue(o1 != o2);
            assertEquals(o1, o2);
            assertEquals(o1.hashCode(), o2.hashCode());

            Hashtable t = new Hashtable();
            t.put(o1, o1);
            t.put(o2, o2);
            assertEquals(1, t.size());
        }
        catch (Exception ex) {}

        try {
            clazz.getDeclaredMethod("equals", new Class[] { Object.class });
            try {
                clazz.getDeclaredMethod("hashCode", new Class[] {});
            }
            catch (NoSuchMethodException ex) {
                fail("Must overwrite #hashCode() in class '" + clazz.getName()
                        + "' when implementing #equals(Object).");
            }

            Object o1 = clazz.newInstance();
            Object o2 = clazz.newInstance();

            assertTrue(o1 != o2);
            assertEquals(o1, o2);
            assertEquals(o1.hashCode(), o2.hashCode());

            Hashtable t = new Hashtable();
            t.put(o1, o1);
            t.put(o2, o2);
            assertEquals(1, t.size());
        }
        catch (Exception ex) {}

    }

    protected int testSerialization(Object o) {
        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            ObjectOutputStream oOut = new ObjectOutputStream(out);

            oOut.writeObject(o);

            oOut.close();

            byte[] data = out.toByteArray();

            ByteArrayInputStream in = new ByteArrayInputStream(data);
            ObjectInputStream oIn = new ObjectInputStream(in);

            oIn.readObject();

            oIn.close();
            return data.length;
        }
        catch (Exception ex) {
            ex.printStackTrace();
            fail(ex.toString());
            return 0;
        }
    }

}
